import os
import sys

from pygments import highlight
from pygments.formatters import HtmlFormatter

current_dir = os.path.dirname(os.path.abspath(__file__))
parent_dir = os.path.dirname(current_dir)
sys.path.append(parent_dir)
import PygmentsVexLexer


lexer = PygmentsVexLexer.VexLexer()
formatter = HtmlFormatter(style='default', full=True, linenos=True)
#formatter = HtmlFormatter(style='monokai', full=True, linenos=True)

with open("./test/test.vfl", "r") as f:
    code = f.read()

with open("./test/test.html", "w") as f:
    f.write( highlight(code, lexer, formatter))


"""
    Pygments Lexer for Side Effects Software's Vex and Vex Wrangle source code.
    Function names were generated for Houdini 21 via the vcc compiler. See
    /syntax/syntax.py for details.
"""
__author__  = "Shawn Lipowski"
__email__   = "vfxcodeblog@gmail.com"
__date__    = "2025-11-16"
__version__ = "1.0.1"
__license__ = "MIT"


import os
import re

from pygments.lexer import RegexLexer, bygroups, words
from pygments.token import Text, Comment, Operator, Keyword, Name, String, \
    Number, Punctuation, Whitespace


_ROOT = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'syntax')

with open(os.path.join(_ROOT, "VexKeywords.txt")) as f:
    VEX_KEYWORDS = f.read().split()
with open(os.path.join(_ROOT, "VexFunctions.txt")) as f:
    VEX_FUNCTIONS = f.read().split()
with open(os.path.join(_ROOT, "VexTypes.txt")) as f:
    VEX_TYPES = f.read().split()
with open(os.path.join(_ROOT, "VexPredefined.txt")) as f:
    VEX_PREDEFINED = f.read().split()
with open(os.path.join(_ROOT, "VexPreprocessor.txt")) as f:
    VEX_PREPROCESSOR = f.read().split()
with open(os.path.join(_ROOT, "VexConstants.txt")) as f:
    VEX_CONSTANTS = f.read().split()
with open(os.path.join(_ROOT, "VexContexts.txt")) as f:
    VEX_CONTEXTS = f.read().split()


class VexLexer(RegexLexer):
    """
    Lexer for Side Effects Software's Vex and Vex Wrangle source.
    """
    name = 'Vex'
    filenames = ['*.vfl', '*.vex', '*.h']
    aliases = ['vfl', 'vex', 'vexwrangle']
    mimetypes = ['text/vex']
    url = 'https://www.sidefx.com/'
    version_added = '1.0.0'

    vex_keywords = (words(VEX_KEYWORDS, suffix=r'\b'), Keyword.Reserved)
    vex_types = (words(VEX_TYPES, suffix=r'\b'), Keyword.Type)
    vex_functions = (words(VEX_FUNCTIONS, suffix=r'\b(\s)*?(\()'), bygroups(Name.Function, Whitespace, Punctuation))
    vex_predefined = (words(VEX_PREDEFINED, suffix=r'\b'), Name.Variable.Magic)
    vex_preprocessor = (words(VEX_PREPROCESSOR, suffix=r'\b'), Comment.Preproc)
    vex_constants = (words(VEX_CONSTANTS, suffix=r'\b'), Name.Constant)
    vex_contexts = (words(VEX_CONTEXTS, suffix=r'\b'), Keyword.Namespace)

    tokens = {
        'root': [
            (r'\n', Whitespace),
            (r'\s+', Whitespace),

            # Comments
            (r'//.*?\n', Comment.Single),
            (r'/\*', Comment.Multiline, 'comment-multiline'),

            # Expression Strings (wrangle string replacement)
            (r'`', String, 'string-backtick'),
            
            # Keywords and Types
            vex_keywords,
            vex_types,
            vex_predefined,
            vex_constants,
            vex_contexts,
            vex_preprocessor,

            # Floats
            (r'[0-9]+\.f', Number.Float),
            (r'[0-9]+\.[0-9]+f', Number.Float),
            (r'[0-9]+\.(?:[0-9][0-9][0-9]_)+[0-9]+', Number.Float),
            (r'[0-9]+\.(?:[eE][-+]?[0-9]+)?', Number.Float),
            (r'[0-9]+\.[0-9]+(?:[eE][-+]?[0-9]+)?', Number.Float),

            # Integers
            (r'0b[01]+', Number.Bin),
            (r'0x[0-9a-fA-F]+', Number.Hex),
            (r'[0-9]+(?:_[0-9][0-9][0-9])+', Number.Integer),
            (r'[0-9]+(?:[eE][-+]?[0-9]+)?', Number.Integer),

            # Wrangle Binding Identifier
            (r'[fuvpi234sd]+(?:\[\])?@[a-zA-Z_]\w*', Name.Variable),
            (r'@[a-zA-Z_]\w*', Name.Variable),

            # Identifier
            (r'[a-zA-Z_]\w*', Name),
            
            # Strings
            (r'"', String, 'string-double'),
            (r"'", String, 'string-single'),

            # Operators, Punctuation
            (r'[+%=><|^!?/\-*&~:@\\]', Operator),
            (r'[{}()\[\],.;]', Punctuation)
        ],
        'string-backtick': [
            (r'`', String, '#pop'),
            (r'[^`]+?', String),
        ],
        'string-double': [
            (r'\\"', String), # escape
            (r'"', String, '#pop'),
            (r'[^"]+?', String),
        ],
        'string-single': [
            (r"\\'", String), # escape
            (r"'", String, '#pop'),
            (r"[^']+?", String),
        ],
        'comment-multiline': [
            (r'\*/', Comment.Multiline, '#pop'),
            (r'[^*]+', Comment.Multiline),
            (r'\*', Comment.Multiline),
        ]
    }

    def get_tokens_unprocessed(self, text, stack=('root',)):

        for index, token, value in RegexLexer.get_tokens_unprocessed(self, text, stack):
            if token is Name:
                if value in VEX_FUNCTIONS:
                    start = index + len(value)
                    if start < len(text):
                        if re.match(r'\s*?\(', text[start:]):
                            token = Name.Function
            yield index, token, value


"""
    syntx.py - Uses Houdini's vcc compiler to query the vex function list.
    First argument to the script is the path to vcc. With no arguments, make
    sure VCC_PATH is pointing to the correct location.

    :author: Shawn Lipowski
    :date: 2025-11-16
    :license: MIT, see LICENSE for details.
"""

import os
import sys
import re
import subprocess

VCC_PATH  = 'C:/Program Files/Side Effects Software/Houdini 21.0.440/bin/vcc.exe'
SYN_PATH  = os.path.join(os.path.dirname(os.path.abspath(__file__)), '../PygmentsVexLexer/syntax')
FUNC_PATH = os.path.join(SYN_PATH, 'VexFunctions.txt')

def contexts(vcc_path=VCC_PATH):
    """Return a sorted list of all vex contexts."""
    ctxs = subprocess.check_output([vcc_path, '-X'])
    ctxs = ctxs.decode('ascii').split('\n')
    return sorted([x for x in ctxs if x != '' and x != None])

def context_functions(context, vcc_path=VCC_PATH, as_set=False):
    """Return the sorted list of all function names for a vex context."""
    ctx_info = subprocess.check_output([vcc_path, '-X', context])
    ctx_info = ctx_info.decode('ascii')

    funcs = set()
    for f in re.findall(r'\w+\(', ctx_info):
        if len(f) > 1:
            funcs.add(f[:-1])

    if as_set:
        return funcs
    else:
        return sorted(funcs)

def all_functions(vcc_path=VCC_PATH, function_path=FUNC_PATH):
    """Returns a sorted list of all vex functions in all contexts."""
    all_funcs = set()
    for ctx in contexts():
        all_funcs.update(context_functions(ctx, as_set=True))

    all_funcs_sorted = sorted(all_funcs)

    with open(function_path, 'w') as out:
        for func in all_funcs_sorted[:-1]:
            out.write(f'{func}\n')
        out.write(f'{all_funcs_sorted[-1]}')

if __name__ == '__main__':
    if len(sys.argv) < 2:
        all_functions()
    elif len(sys.argv) == 2:
        all_functions(sys.argv[1])
    else:
        raise Exception('To many arguments.')